part of 'bolter_provider.dart';

class BolterBuilder<T> extends StatefulWidget {
  final T Function()? getter;
  final BolterBuilderController<T>? controller;
  final Widget Function(BuildContext context, T value) builder;

  const BolterBuilder({
    super.key,
    required this.getter,
    required this.builder,
  }) : controller = null;

  const BolterBuilder.controller({
    super.key,
    required this.controller,
    required this.builder,
  }) : getter = null;

  @override
  _BolterBuilderState<T> createState() => _BolterBuilderState<T>();
}

class _BolterBuilderState<T> extends State<BolterBuilder<T>> {
  late final _bolter = context.bolter;

  Getter? get _getter => widget.getter;
  BolterBuilderController<T>? get _controller => widget.controller;

  void _notification() {
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    _controller?._notifications.add(_notification);
    _bolter.listen(_getter ?? () => widget.controller!._value, _notification);
  }

  @override
  void didUpdateWidget(BolterBuilder<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.getter != widget.getter ||
        oldWidget.controller != widget.controller) {
      _bolter.stopListen(_notification);
      _bolter.listen(_getter ?? () => _controller!._value, _notification);
    }
  }

  @override
  Widget build(BuildContext context) {
    final getter = _getter;
    return widget.builder(
      context,
      getter != null ? getter() : _controller!._value,
    );
  }

  @override
  void dispose() {
    _controller?._notifications.remove(_notification);
    _bolter.stopListen(_notification);
    super.dispose();
  }
}

class BolterBuilderController<T> {
  final BolterInterface bolter;
  late T _value;
  final _notifications = <BolterNotification>[];

  BolterBuilderController(this.bolter, this._value);

  T get value => _value;

  void update(T Function(T value) action) {
    for (final notification in _notifications) {
      bolter.runAndUpdate(
        action: () => _value = action(_value),
        exactNotification: notification,
      );
    }
  }

  void dispose() {
    _notifications.clear();
  }
}
